//
//  LoadController.m
//  FTS
//
//  Created by Michael Heyeck on 2/12/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "Text.h"
#import "FTSAppDelegate.h"
#import "LoadController.h"


@interface LoadController ()

@property (nonatomic, retain) NSDate* start;

- (void)doLoadWithCoordinator:(NSPersistentStoreCoordinator*)psc;
- (void)updateProgress:(NSArray*)progress;

@end


@implementation LoadController

@synthesize start;
@synthesize target;
@synthesize action;
@synthesize label;
@synthesize activityView;
@synthesize progressView;


/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)startLoadWithTarget:(id)a_target action:(SEL)a_action
{
	// Record callback info
	self.target = a_target;
	self.action = a_action;

	// Grab the psc (on the main thread, as the getter isn't threadsafe)
	NSPersistentStoreCoordinator* psc = [(FTSAppDelegate*)[[UIApplication sharedApplication] delegate] persistentStoreCoordinator];
    if (!psc)
	{
		NSLog(@"Load failed:  Could not get NSPersistentStoreCoordinator");
		return;
	}

	// Place overlay on the main window
	self.view.frame = [[UIScreen mainScreen] applicationFrame];
	[[(FTSAppDelegate*)[[UIApplication sharedApplication] delegate] window] addSubview:self.view];

	// Record start time
	self.start = [NSDate date];
	
	// Launch helper thread (self retained until thread terminates)
	[self performSelectorInBackground:@selector(doLoadWithCoordinator:) withObject:psc];
}


- (void)dealloc
{
	[start release];
	[label release];
	[activityView release];
	[progressView release];
    [super dealloc];
}

#pragma mark Extension methods

- (void)doLoadWithCoordinator:(NSPersistentStoreCoordinator*)psc
{
	// Setup pool
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

	// Setup MOC
	NSManagedObjectContext* moc = [[[NSManagedObjectContext alloc] init] autorelease];
	moc.persistentStoreCoordinator = psc;

	// Find pathname of data to load
	NSString* path = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"civ.plist"];

	// Load records
	NSArray* a = [NSArray arrayWithContentsOfFile:path];
	NSUInteger t = 0; for (NSString* s in a) t += [s length]; NSNumber* ot = [NSNumber numberWithUnsignedInteger:t];

	// Initialize progress screen
	NSArray* p = [NSArray arrayWithObjects:ot,[NSNumber numberWithUnsignedInteger:0],nil];
	[self performSelectorOnMainThread:@selector(updateProgress:) withObject:p waitUntilDone:NO];

    // Index and commit records to DB
	// Note that it's about 30% slower to perform commits for each record than to perform a single
	// commit at the end.  However, if we do a single commit we can't provide user feedback on progress;
	// the commit takes 95%+ of the execution time on the device, and Core Data doesn't provide any
	// feedback on how it's going.  This is a weird case where making the code more "responsive" actually
	// makes it slower.
	NSError* e = nil; NSUInteger c = 0;
	for (NSString* s in a)
	{
		NSAutoreleasePool* ip = [[NSAutoreleasePool alloc] init];
		
		[[NSEntityDescription insertNewObjectForEntityForName:@"Text" inManagedObjectContext:moc] setText:s];

		//if (![moc save:&e]) NSLog(@"Unresolved error %@, %@", e, [e userInfo]);

		c += [s length];
		//NSArray* p = [NSArray arrayWithObjects:ot,[NSNumber numberWithUnsignedInteger:c],nil];
		//[self performSelectorOnMainThread:@selector(updateProgress:) withObject:p waitUntilDone:NO];
		
		[ip release];
	}
	// I'm choosing reality over appearances, though I'm not entirely happy about it
	if (![moc save:&e]) NSLog(@"Unresolved error %@, %@", e, [e userInfo]);
	p = [NSArray arrayWithObjects:ot,[NSNumber numberWithUnsignedInteger:c],nil];
	[self performSelectorOnMainThread:@selector(updateProgress:) withObject:p waitUntilDone:NO];

	// Teardown pool
	[pool release];
}


- (void)updateProgress:(NSArray*)progress
{
	// Get the data required to populate the progress screen
	NSTimeInterval e = [[NSDate date] timeIntervalSinceDate:self.start];
	float t = (float) [[progress objectAtIndex:0] unsignedIntegerValue];
	float c = (float) [[progress objectAtIndex:1] unsignedIntegerValue];
	
	// Estimate time remaining; use actual data if possible, or default to a rule-of-thumb
	if (c > 0.0f)
	{
		float r = e*(t-c)/c;
		NSString* f = @"Indexing %d bytes\nEstimated time remaining: %.1fs";
		label.text = [NSString stringWithFormat:f,(int)t,r];
	}
	else
	{
		float r = 65.0*t/175500.0;
		NSString* f = @"Indexing %d bytes\n(This should take about %.0fs)";
		label.text = [NSString stringWithFormat:f,(int)t,r];
	}

	// Update progress bar
	progressView.progress = c/t;

	// Upon completion, dismiss the progress screen
	if (c >= t)
	{
		[self.view removeFromSuperview];
		[target performSelector:action];
	}
}

@end
