// 
//  Text.m
//  FTS
//
//  Created by Michael Heyeck on 2/3/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "Text.h"
#import "Keyword.h"
#import "Tokenizer.h"


@interface Text (OtherAccessors)

@property (nonatomic, retain) NSString* sortPrefix;

- (void)setPrimitiveText:(NSString*)newText;

@end


@implementation Text 

@dynamic text;
@dynamic words;


- (void)setText:(NSString*)newText
{
	[self willChangeValueForKey:@"text"];
    [self setPrimitiveText:newText];
    [self didChangeValueForKey:@"text"];
	
	NSUInteger l = [newText length];
	self.sortPrefix = [newText substringToIndex:MIN(l, 20)];
}


- (void)willSave
{
	// Superclass
	[super willSave];

	// Omit keyword processing for deleted objects
	if (self.isDeleted || computedKeywords) return;

	// Process this object for keywords
	NSMutableSet* tokens = [[Tokenizer sharedTokenizer] tokenize:self.text];

	// Create and perform a request for existing keyword records for these tokens
	NSError* err;
	NSFetchRequest* request = [[NSFetchRequest alloc] init];
	request.entity = [NSEntityDescription entityForName:@"Keyword" inManagedObjectContext:self.managedObjectContext];
	request.predicate = [NSPredicate predicateWithFormat:@"keyword IN %@",tokens];
	NSMutableSet* keywords = [NSMutableSet setWithArray:[self.managedObjectContext executeFetchRequest:request error:&err]];
	[request release];

	// Find nonexistent keywords (by removing existing keywords from tokens) ...
	[tokens minusSet:[keywords valueForKey:@"keyword"]];
	// ... then create them
	for (id k in tokens)
	{
		NSManagedObject* newManagedObject = [NSEntityDescription insertNewObjectForEntityForName:@"Keyword" inManagedObjectContext:self.managedObjectContext];
		[newManagedObject setValue:k forKey:@"keyword"];
		[keywords addObject:newManagedObject];
	}
	
	// Make changes
	NSMutableSet* keywordsToAdd = [[keywords mutableCopy] autorelease];
	NSMutableSet* keywordsToRemove = [[self.words mutableCopy] autorelease];
	[keywordsToAdd minusSet:self.words];
	[keywordsToRemove minusSet:keywords];
	if ([keywordsToAdd count]) [self addWords:keywordsToAdd];
	if ([keywordsToRemove count]) [self removeWords:keywordsToRemove];

	// Flag object as ready for saving
	// Note that this assumes the object WILL NOT BE CHANGED between this point and the commit to DB
	computedKeywords = YES;
}


- (void)didSave
{
	computedKeywords = NO;
}

@end
