//
//  Tokenizer.m
//  FTS
//
//  Created by Michael Heyeck on 2/3/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "Tokenizer.h"


// For singleton implementation
static Tokenizer* __tokenizer = nil;


@implementation Tokenizer

+ (Tokenizer*)sharedTokenizer
{
	if (!__tokenizer)
	{
		__tokenizer = [[Tokenizer alloc] init];
	}
	return __tokenizer;
}


// Returns the set of stop words for FTS (from MySQL)
- (NSSet*)stopWords
{
	if (stopWords) return stopWords;
	
	NSString* path = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"stopwords.plist"];
	return (stopWords = [[NSSet alloc] initWithArray:[NSArray arrayWithContentsOfFile:path]]);
}


- (NSCharacterSet*)splitChars
{
	if (splitChars) return splitChars;

	NSCharacterSet* cs = [NSCharacterSet characterSetWithCharactersInString:@"abcdefghijklmnopqrstuvwxyz0123456789'"];
	return (splitChars = [[cs invertedSet] retain]);
}


- (NSMutableSet*)tokenize:(NSString*)string
{
	// Remove diacritics and convert to lower case
	string = [string stringByFoldingWithOptions:kCFCompareCaseInsensitive|kCFCompareDiacriticInsensitive locale:[NSLocale systemLocale]];

	// Split on [^a-z0-9']
	NSArray* a = [string componentsSeparatedByCharactersInSet:[self splitChars]];

	// Convert to a set, remove stopwords (including the empty string), and return
	NSMutableSet* s = [NSMutableSet setWithArray:a];
	[s minusSet:[self stopWords]];
	return s;
}


- (NSString*)findLastToken:(NSString*)string position:(NSInteger*)position
{
	// Default
	if (position) *position = NSNotFound;
	
	// Remove diacritics and convert to lower case
	string = [string stringByFoldingWithOptions:kCFCompareCaseInsensitive|kCFCompareDiacriticInsensitive locale:[NSLocale systemLocale]];
	
	// Split on [^a-z0-9']
	NSArray* a = [string componentsSeparatedByCharactersInSet:[self splitChars]];

	// Find the position of the last non-empty keyword candidate
	for (NSInteger i = [a count]-1; i >= 0; i--)
	{
		NSString* s = [a objectAtIndex:i];
		if (![s length]) continue;
		
		if (position) *position = [string rangeOfString:s options:NSBackwardsSearch].location;
		return s;
	}
	
	// Default
	return nil;
}


- (void)dealloc
{
	[stopWords release];
	[splitChars release];
	[super dealloc];
}

@end
